package main

import (
	"bytes"
	"fmt"
	"io"
	"log"
	"net/http"
	"time"
)

// Home Page, 实现 handler
func home(w http.ResponseWriter, r *http.Request) {
	//n, err := resp.Write([]byte("home page"))
	n, err := io.WriteString(w, "home page")
	if err != nil {
		fmt.Println(err.Error())
		return
	}
	fmt.Println("send to client", n, "bytes.")
}

// Post 处理器，实现 http.Handler interface
type Post struct{}

func (p Post) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	io.WriteString(w, time.Now().Format("2006-01-02 15:04:05"))
}

type Info struct{}

func (info Info) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	fmt.Printf("%#v\n", r.URL)

	fmt.Println(r.RemoteAddr)

	w.WriteHeader(200)
	_, _ = io.WriteString(w, fmt.Sprintf("%s %s %s\r\n", r.Method, r.RequestURI, r.Proto))

	for k, v := range r.Header {
		_, _ = io.WriteString(w, fmt.Sprintf("%s: %s\r\n", k, v))
	}

	io.WriteString(w, "\r\n")

	fmt.Println("req body is:")
	//bs := make([]byte, 1024)
	//for {
	//	n, err := r.Body.Read(bs)
	//	if n != 0 {
	//		fmt.Print(string(bs[:n]))
	//		_, _ = w.Write(bs[:n])
	//	}
	//	if err != nil {
	//		if err != io.EOF {
	//			fmt.Println(err.Error())
	//		}
	//		break
	//	}
	//}
	//io.Copy(os.Stdout, r.Body)
	//io.Copy(w, r.Body)
	buf := bytes.NewBuffer(nil)
	io.Copy(buf, r.Body)
	fmt.Println(buf.String())
	io.Copy(w, buf)
}

func main() {
	addr := "0.0.0.0:8000"

	// 面向过程实现
	http.HandleFunc("/", home)

	// 面向对象实现
	//http.Handle("/post/", Post{})
	http.Handle("/post/", &Post{})

	// curl -XPOST http://0.0.0.0:8000/info -d "hello world"
	http.Handle("/info", &Info{})

	fmt.Println("start server, listen on", addr)
	log.Fatal(http.ListenAndServe(addr, nil))
}
